@@ -89,10 +89,21 @@ class ApplianceImageCreator(ImageCreator):
                     p = p1
                     break
 
-            s +=  "%(device)s  %(mountpoint)s %(fstype)s    defaults,noatime 0 0\n" %  {
-                'device': "/dev/%s%-d" % (p['disk'], p['num']),
-                'mountpoint': p['mountpoint'],
-                'fstype': p['fstype'] }
+            s += "%(device)s  %(mountpoint)s         %(fstype)s   %(fsopts)s 0 0\n" %  {
+                 'device': "/dev/%s%-d" % (p['disk'], p['num']),
+                 'mountpoint': p['mountpoint'],
+                 'fstype': p['fstype'],
+                 'fsopts': "defaults,noatime" if not p['fsopts'] else p['fsopts']}
+
+            if p['mountpoint'] == "/":
+                for subvol in self.__instloop.subvolumes:
+                    if subvol['mountpoint'] == "/":
+                        continue
+                    s += "%(device)s  %(mountpoint)s         %(fstype)s   %(fsopts)s 0 0\n" %  {
+                         'device': "/dev/%s%-d" % (p['disk'], p['num']),
+                         'mountpoint': subvol['mountpoint'],
+                         'fstype': p['fstype'],
+                         'fsopts': "defaults,noatime" if not subvol['fsopts'] else subvol['fsopts']}
 
         s += "devpts     /dev/pts  devpts  gid=5,mode=620   0 0\n"
         s += "tmpfs      /dev/shm  tmpfs   defaults         0 0\n"
@@ -167,7 +178,7 @@ class ApplianceImageCreator(ImageCreator):
                                            self._instroot)
 
         for p in parts:
-            self.__instloop.add_partition(int(p.size), p.disk, p.mountpoint, p.fstype, p.active)
+            self.__instloop.add_partition(int(p.size), p.disk, p.mountpoint, p.fstype, fsopts = p.fsopts, boot = p.active)
 
         try:
             self.__instloop.mount()
