@@ -24,6 +24,7 @@ import glob
 import shutil
 import subprocess
 import logging
+import time
 
 from mic.imgcreate.errors import *
 from mic.imgcreate.fs import *
@@ -52,10 +53,11 @@ class PartitionedMount(Mount):
         self.parted=find_binary_path("parted")
         self.kpartx=find_binary_path("kpartx")
         self.mkswap=find_binary_path("mkswap")
-        self.btrfscmd=find_binary_path("btrfs")
+        self.btrfscmd=None
         self.mountcmd=find_binary_path("mount")
         self.umountcmd=find_binary_path("umount")
         self.skipformat = skipformat
+        self.snapshot_created = self.skipformat
         # Size of a sector used in calculations
         self.sector_size = 512
 
@@ -65,6 +67,7 @@ class PartitionedMount(Mount):
 
         """ We need to handle subvolumes for btrfs """
         if fstype == "btrfs" and fsopts and fsopts.find("subvol=") != -1:
+            self.btrfscmd=find_binary_path("btrfs")
             subvol = None
             opts = fsopts.split(",")
             for opt in opts:
@@ -349,6 +352,9 @@ class PartitionedMount(Mount):
 
             if p['mount'] != None:
                 try:
+                    """ Create subvolume snapshot here """
+                    if p['fstype'] == "btrfs" and p['mountpoint'] == "/" and not self.snapshot_created:
+                        self.__create_subvolume_snapshots(p, p["mount"])
                     p['mount'].cleanup()
                 except:
                     pass
@@ -356,6 +362,8 @@ class PartitionedMount(Mount):
 
     """ Only for btrfs """
     def __get_subvolume_id(self, rootpath, subvol):
+        if not self.btrfscmd:
+            self.btrfscmd=find_binary_path("btrfs")
         argv = [ self.btrfscmd, "subvolume", "list", rootpath ]
         p1 = subprocess.Popen(argv, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
         (out,err) = p1.communicate()
@@ -373,6 +381,8 @@ class PartitionedMount(Mount):
         return subvolid
 
     def __create_subvolume_metadata(self, p, pdisk):
+        if len(self.subvolumes) == 0:
+            return
         argv = [ self.btrfscmd, "subvolume", "list", pdisk.mountdir ]
         p1 = subprocess.Popen(argv, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
         (out,err) = p1.communicate()
@@ -402,6 +412,8 @@ class PartitionedMount(Mount):
 
     def __get_subvolume_metadata(self, p, pdisk):
         subvolume_metadata_file = "%s/.subvolume_metadata" % pdisk.mountdir
+        if not os.path.exists(subvolume_metadata_file):
+            return
         fd = open(subvolume_metadata_file, "r")
         content = fd.read()
         fd.close()
@@ -490,7 +502,8 @@ class PartitionedMount(Mount):
                 if opt.strip().startswith("subvol="):
                     opts.remove(opt)
                     break
-            opts.append("subvolid=%d" % subvolid)
+            #opts.append("subvolid=%d" % subvolid)
+            opts.extend(["subvolid=0", "subvol=%s" % subvol["subvol"]])
             fsopts = ",".join(opts)
             subvol['fsopts'] = fsopts
             mountpoint = self.mountdir + subvol['mountpoint']
@@ -512,6 +525,35 @@ class PartitionedMount(Mount):
             if rc != 0:
                 raise MountError("Failed to unmount subvolume %s from %s" % (subvol["subvol"], mountpoint))
             subvol["mounted"] = False
+
+    def __create_subvolume_snapshots(self, p, pdisk):
+        if self.snapshot_created:
+            return
+
+        """ Remount with subvolid=0 """
+        rc = subprocess.call([self.umountcmd, pdisk.mountdir])
+        if rc != 0:
+            raise MountError("Failed to umount %s" % pdisk.mountdir)
+        if pdisk.fsopts:
+            mountopts = pdisk.fsopts + ",subvolid=0"
+        else:
+            mountopts = "subvolid=0"
+        rc = subprocess.call([self.mountcmd, "-o", mountopts, pdisk.disk.device, pdisk.mountdir])
+        if rc != 0:
+            raise MountError("Failed to umount %s" % pdisk.mountdir)
+
+        """ Create all the subvolume snapshots """
+        snapshotts = time.strftime("%Y%m%d-%H%M")
+        for subvol in self.subvolumes:
+            subvolpath = pdisk.mountdir + "/" + subvol["subvol"]
+            snapshotpath = subvolpath + "_%s-1" % snapshotts
+            argv = [ self.btrfscmd, "subvolume", "snapshot", subvolpath, snapshotpath ]
+            p1 = subprocess.Popen(argv, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+            (out,err) = p1.communicate()
+            logging.debug(out)
+            if p1.returncode != 0:
+                raise MountError("Failed to create subvolume snapshot '%s' for '%s', return code: %d." % (snapshotpath, subvolpath, p1.returncode))
+        self.snapshot_created = True
   
     def mount(self):
         for dev in self.disks.keys():
