@@ -147,7 +147,10 @@ def setup_chrootenv(chrootdir, bindmounts = None):
     globalmounts = get_bind_mounts(chrootdir, bindmounts)
     bind_mount(globalmounts)
     setup_resolv(chrootdir)
-    shutil.copyfile("/etc/mtab", chrootdir + "/etc/mtab")
+    mtab = "/etc/mtab"
+    dstmtab = chrootdir + mtab
+    if not os.path.islink(dstmtab):
+        shutil.copyfile(mtab, dstmtab)
     chroot_lock = os.path.join(chrootdir, ".chroot.lock")
     chroot_lockfd = open(chroot_lock, "w")
     return globalmounts
