@@ -665,19 +665,31 @@ class ImageCreator(object):
 
         """
         try:
-            os.unlink(self._instroot + "/etc/mtab")
+            mtab = self._instroot + "/etc/mtab"
+            if not os.path.islink(mtab):
+                os.unlink(self._instroot + "/etc/mtab")
             if self.qemu_emulator:
                 os.unlink(self._instroot + self.qemu_emulator)
-            """ Clean up yum garbage """
-            instroot_pdir = os.path.dirname(self._instroot + self._instroot)
-            if os.path.exists(instroot_pdir):
-                shutil.rmtree(instroot_pdir, ignore_errors = True)
         except OSError:
             pass
 
 
         self._undo_bindmounts()
 
+        """ Clean up yum garbage """
+        try:
+            instroot_pdir = os.path.dirname(self._instroot + self._instroot)
+            if os.path.exists(instroot_pdir):
+                shutil.rmtree(instroot_pdir, ignore_errors = True)
+            yumcachedir = self._instroot + "/var/cache/yum"
+            if os.path.exists(yumcachedir):
+                shutil.rmtree(yumcachedir, ignore_errors = True)
+            yumlibdir = self._instroot + "/var/lib/yum"
+            if os.path.exists(yumlibdir):
+                shutil.rmtree(yumlibdir, ignore_errors = True)
+        except OSError:
+            pass
+
         self._unmount_instroot()
 
     def cleanup(self):
