@@ -52,13 +52,13 @@ class TextProgress(object):
     def start(self, filename, url, *args, **kwargs):
         self.url = url
         self.termwidth = terminal_width()
-        sys.stdout.write("Retrieving %s " % truncate_url(self.url, self.termwidth - 13))
+        sys.stdout.write("Retrieving %s " % truncate_url(self.url, self.termwidth - 17))
         sys.stdout.flush()
         self.indicators = ["-", "\\", "|", "/"]
         self.counter = 0
     def update(self, *args):
         if sys.stdout.isatty():
-            sys.stdout.write("\rRetrieving %s %s" % (truncate_url(self.url, self.termwidth - 13), self.indicators[self.counter%4]))
+            sys.stdout.write("\rRetrieving %s %s" % (truncate_url(self.url, self.termwidth - 17), self.indicators[self.counter%4]))
             sys.stdout.flush()
             self.counter += 1
         else:
@@ -179,7 +179,7 @@ class BindChrootMount:
             raise MountError("Bind-mounting '%s' to '%s' failed" %
                              (self.src, self.dest))
         if self.option:
-            rc = subprocess.call([self.mountcmd, "-o", "remount,%s" % self.option, self.dest])
+            rc = subprocess.call([self.mountcmd, "--bind", "-o", "remount,%s" % self.option, self.dest])
             if rc != 0:
                 raise MountError("Bind-remounting '%s' failed" % self.dest)
         self.mounted = True
@@ -487,12 +487,13 @@ class DiskMount(Mount):
 
 class ExtDiskMount(DiskMount):
     """A DiskMount object that is able to format/resize ext[23] filesystems."""
-    def __init__(self, disk, mountdir, fstype, blocksize, fslabel, rmmountdir=True, skipformat = False):
+    def __init__(self, disk, mountdir, fstype, blocksize, fslabel, rmmountdir=True, skipformat = False, fsopts = None):
         DiskMount.__init__(self, disk, mountdir, fstype, rmmountdir)
         self.blocksize = blocksize
         self.fslabel = fslabel.replace("/", "")
         self.uuid  = None
         self.skipformat = skipformat
+        self.fsopts = fsopts
         self.dumpe2fs = find_binary_path("dumpe2fs")
         self.tune2fs = find_binary_path("tune2fs")
 
@@ -508,11 +509,14 @@ class ExtDiskMount(DiskMount):
             logging.debug("Skip filesystem format.")
             return
         logging.debug("Formating %s filesystem on %s" % (self.fstype, self.disk.device))
-        rc = subprocess.call([self.mkfscmd,
+        proc = subprocess.Popen([self.mkfscmd,
                               "-F", "-L", self.fslabel,
                               "-m", "1", "-b", str(self.blocksize),
-                              self.disk.device], stdout=sys.stdout,
-                              stderr=sys.stderr)
+                              self.disk.device], stdout=subprocess.PIPE,
+                              stderr=subprocess.PIPE)
+        rc = proc.wait()
+        output = proc.communicate()[0].replace('\b', ' ')
+        sys.stdout.write(output.decode('UTF-8'))
         #                      str(self.disk.size / self.blocksize)])
         if rc != 0:
             raise MountError("Error creating %s filesystem on disk %s" % (self.fstype,self.disk.device))
@@ -606,12 +610,13 @@ class ExtDiskMount(DiskMount):
 
 class VfatDiskMount(DiskMount):
     """A DiskMount object that is able to format vfat/msdos filesystems."""
-    def __init__(self, disk, mountdir, fstype, blocksize, fslabel, rmmountdir=True, skipformat = False):
+    def __init__(self, disk, mountdir, fstype, blocksize, fslabel, rmmountdir=True, skipformat = False, fsopts = None):
         DiskMount.__init__(self, disk, mountdir, fstype, rmmountdir)
         self.blocksize = blocksize
         self.fslabel = fslabel.replace("/", "")
         self.uuid = "%08X" % int(time.time())
         self.skipformat = skipformat
+        self.fsopts = fsopts
         self.fsckcmd = find_binary_path("fsck." + self.fstype)
 
     def __format_filesystem(self):
@@ -685,13 +690,14 @@ class VfatDiskMount(DiskMount):
 
 class BtrfsDiskMount(DiskMount):
     """A DiskMount object that is able to format/resize btrfs filesystems."""
-    def __init__(self, disk, mountdir, fstype, blocksize, fslabel, rmmountdir=True, skipformat = False):
+    def __init__(self, disk, mountdir, fstype, blocksize, fslabel, rmmountdir=True, skipformat = False, fsopts = None):
         self.__check_btrfs()
         DiskMount.__init__(self, disk, mountdir, fstype, rmmountdir)
         self.blocksize = blocksize
         self.fslabel = fslabel.replace("/", "")
         self.uuid  = None
         self.skipformat = skipformat
+        self.fsopts = fsopts
         self.blkidcmd = find_binary_path("blkid")
         self.btrfsckcmd = find_binary_path("btrfsck")
 
@@ -706,10 +712,6 @@ class BtrfsDiskMount(DiskMount):
         if not found:
             raise MountError("Your system can't mount btrfs filesystem, please make sure your kernel has btrfs support and the module btrfs.ko has been loaded.")
 
-        # disable selinux, selinux will block write
-        if os.path.exists("/usr/sbin/setenforce"):
-            subprocess.call(["/usr/sbin/setenforce", "0"])
-
     def __parse_field(self, output, field):
         for line in output.split(" "):
             if line.startswith(field + "="):
@@ -765,8 +767,6 @@ class BtrfsDiskMount(DiskMount):
 
     def mount(self, options = None):
         self.__create()
-        if not options:
-            options = "compress"
         DiskMount.mount(self, options)
 
     def __fsck(self):
