@@ -297,12 +297,13 @@ class XConfig(KickstartConfig):
     """A class to apply a kickstart X configuration to a system."""
     def apply(self, ksxconfig):
         if ksxconfig.startX:
-            f = open(self.path("/etc/inittab"), "rw+")
-            buf = f.read()
-            buf = buf.replace("id:3:initdefault", "id:5:initdefault")
-            f.seek(0)
-            f.write(buf)
-            f.close()
+            if os.path.exists(self.path("/etc/inittab")):
+                f = open(self.path("/etc/inittab"), "rw+")
+                buf = f.read()
+                buf = buf.replace("id:3:initdefault", "id:5:initdefault")
+                f.seek(0)
+                f.write(buf)
+                f.close()
         if ksxconfig.defaultdesktop:
             f = open(self.path("/etc/sysconfig/desktop"), "w")
             f.write("DESKTOP="+ksxconfig.defaultdesktop+"\n")
@@ -639,12 +640,12 @@ def get_kernel_args(ks, default = "ro liveimg"):
         return default
     return "%s %s" %(default, ks.handler.bootloader.appendLine)
 
-def get_menu_args(ks, default = "bootinstall"):
+def get_menu_args(ks, default = "liveinst"):
     if not hasattr(ks.handler.bootloader, "menus"):
         return default
-    if ks.handler.bootloader.menus is None:
+    if not ks.handler.bootloader.menus:
         return default
-    return "%s %s" %(default, ks.handler.bootloader.menus)
+    return "%s" % ks.handler.bootloader.menus
 
 def get_default_kernel(ks, default = None):
     if not hasattr(ks.handler.bootloader, "default"):
