@@ -353,7 +353,7 @@ class x86LiveImageCreator(LiveImageCreatorBase):
                  "-boot-load-size", "4" ]
 
     def _get_required_packages(self):
-        return ["syslinux", "syslinux-extlinux", "moblin-live"] + LiveImageCreatorBase._get_required_packages(self)
+        return ["syslinux", "syslinux-extlinux"] + LiveImageCreatorBase._get_required_packages(self)
 
     def _get_isolinux_stanzas(self, isodir):
         return ""
@@ -481,35 +481,45 @@ menu color cmdline 0 #ffffffff #00000000
             raise CreatorError("Unable to find valid kernels, please check the repo")
 
         kernel_options = self._get_kernel_options()
-        menu_options = self._get_menu_options()
 
+        """ menu can be customized highly, the format is
+
+              short_name1:long_name1:extra_options1;short_name2:long_name2:extra_options2
+
+            for example: autoinst:Installation only:systemd.unit=installer-graphical.service
+            but in order to keep compatible with old format, these are still ok:
+
+              liveinst autoinst
+              liveinst;autoinst
+              liveinst::;autoinst::
+        """
+        oldmenus = {"basic":{"short":"basic", "long":"Installation Only (Text based)", "extra":"basic nosplash 4"},
+                    "liveinst":{"short":"liveinst", "long":"Installation Only", "extra":"liveinst nosplash 4"},
+                    "autoinst":{"short":"autoinst", "long":"Autoinstall (Deletes all existing content)", "extra":"autoinst nosplash 4"},
+                    "netinst":{"short":"netinst", "long":"Network Installation", "extra":"netinst 4"},
+                    "verify":{"short":"check", "long":"Verify and", "extra":"check"}
+                   }
+        menu_options = self._get_menu_options()
+        menus = menu_options.split(";")
+        for i in range(len(menus)):
+            menus[i] = menus[i].split(":")
+        if len(menus) == 1 and len(menus[0]) == 1:
+            """ Keep compatible with the old usage way """
+            menus = menu_options.split()
+            for i in range(len(menus)):
+                menus[i] = [menus[i]]
 
         cfg = ""
 
         default_version = None
         default_index = None
         index = "0"
+        netinst = None
         for version in versions:
             is_xen = self.__copy_kernel_and_initramfs(isodir, version, index)
 
             default = self.__is_default_kernel(kernel, kernels)
-            liveinst = False
-            autoliveinst = False
-            netinst = False
-            checkisomd5 = False
-            basicinst = False
             
-            if menu_options.find("bootinstall") >= 0:
-                liveinst = True
-            
-            if menu_options.find("autoinst") >= 0:
-                autoliveinst = True
-                
-            if menu_options.find("verify") >= 0 and self._has_checkisomd5():
-                checkisomd5 = True 
-                               
-            if menu_options.find("netinst") >= 0:
-                netinst = True 
                 
             if default:
                 long = "Boot %s" % self.distro_name
@@ -517,6 +527,7 @@ menu color cmdline 0 #ffffffff #00000000
                 long = "Boot %s(%s)" % (self.name, kernel[7:])
             else:
                 long = "Boot %s(%s)" % (self.name, kernel)
+            oldmenus["verify"]["long"] = "%s %s" % (oldmenus["verify"]["long"], long)
 
             cfg += self.__get_image_stanza(is_xen,
                                            fslabel = self.fslabel,
@@ -530,39 +541,36 @@ menu color cmdline 0 #ffffffff #00000000
                 cfg += "menu default\n"
                 default_version = version
                 default_index = index
-            if basicinst:
-                cfg += self.__get_image_stanza(is_xen,
-                                               fslabel = self.fslabel,
-                                               liveargs = kernel_options,
-                                               long = "Installation Only (Text based)",
-                                               short = "basic" + index,
-                                               extra = "basic nosplash 4",
-                                               index = index)
-                
-            if liveinst:
-                cfg += self.__get_image_stanza(is_xen,
-                                               fslabel = self.fslabel,
-                                               liveargs = kernel_options,
-                                               long = "Installation Only",
-                                               short = "liveinst" + index,
-                                               extra = "liveinst nosplash 4",
-                                               index = index)
-            if autoliveinst:
-                cfg += self.__get_image_stanza(is_xen,
-                                               fslabel = self.fslabel,
-                                               liveargs = kernel_options,
-                                               long = "Autoinstall (Deletes all existing content)",
-                                               short = "autoinst" + index,
-                                               extra = "autoinst nosplash 4",
-                                               index = index)
 
-            if checkisomd5:
+            for menu in menus:
+                if not menu[0]:
+                    continue
+                short = menu[0] + index
+
+                if len(menu) >= 2:
+                    long = menu[1]
+                else:
+                    if menu[0] in oldmenus.keys():
+                        if menu[0] == "verify" and not self._has_checkisomd5():
+                            continue
+                        if menu[0] == "netinst":
+                            netinst = oldmenus[menu[0]]
+                            continue
+                        long = oldmenus[menu[0]]["long"]
+                        extra = oldmenus[menu[0]]["extra"]
+                    else:
+                        long = short.upper() + " X" + index
+                        extra = ""
+
+                if len(menu) >= 3:
+                    extra = menu[2]
+
                 cfg += self.__get_image_stanza(is_xen,
                                                fslabel = self.fslabel,
                                                liveargs = kernel_options,
-                                               long = "Verify and " + long,
-                                               short = "check" + index,
-                                               extra = "check",
+                                               long = long,
+                                               short = short,
+                                               extra = extra,
                                                index = index)
 
             index = str(int(index) + 1)
@@ -572,14 +580,13 @@ menu color cmdline 0 #ffffffff #00000000
         if not default_index:
             default_index = "0"
 
-        
         if netinst:
             cfg += self.__get_image_stanza(is_xen,
                                            fslabel = self.fslabel,
                                            liveargs = kernel_options,
-                                           long = "Network Installation",
-                                           short = "netinst",
-                                           extra = "netinst 4",
+                                           long = netinst["long"],
+                                           short = netinst["short"],
+                                           extra = netinst["extra"],
                                            index = default_index)
 
         return cfg
