@@ -397,7 +397,7 @@ class Yum(yum.YumBase):
         if self.__recording_pkgs:
             # record all pkg and the content
             for pkg in dlpkgs:
-                pkg_long_name = "%s-%s.%s.rpm" % (pkg.name, pkg.printVer(), pkg.arch)
+                pkg_long_name = pkg.compactPrint()
                 self.__pkgs_content[pkg_long_name] = pkg.files
 
         total_count = len(dlpkgs)
