@@ -59,6 +59,7 @@ class Zypp:
         self.bin_rpm = find_binary_path("rpm")
         self.incpkgs = []
         self.excpkgs = []
+        self.pkgstable = {}
 
     def doFileLogSetup(self, uid, logfile):
         # don't do the file log for the livecd as it can lead to open fds
@@ -114,53 +115,47 @@ class Zypp:
         self.installroot = installroot
 
     def selectPackage(self, pkg):
-        """ Select a given package or package pattern, can be specified with name.arch or name* or *name """
+        """Select a given package.  Can be specified with name.arch or name*"""
+        def markPoolItem(name, poolitem):
+            if name not in self.packages:
+                self.packages.append(name)
+                poolitem.status().setToBeInstalled (zypp.ResStatus.USER)
+
         if not self.Z:
             self.__initialize_zypp()
-        
+
         found = False
         startx = pkg.startswith("*")
         endx = pkg.endswith("*")
         ispattern = startx or endx
         sp = pkg.rsplit(".", 2)
-        for item in self.Z.pool():
-            kind = "%s" % item.kind()
-            if kind == "package":
-                name = "%s" % item.name()
-                if not ispattern:
-                    if name in self.incpkgs or self.excpkgs:
-                        found = True
-                        break
-                    if len(sp) == 2:
+        name = sp[0]
+        if not ispattern:
+            try:
+                value = self.pkgstable[name]
+                if len(sp) == 2:
+                    for ver, item in value:
                         arch = "%s" % item.arch()
-                        if name == sp[0] and arch == sp[1]:
+                        if arch == sp[1]:
                             found = True
-                            if name not in self.packages:
-                                self.packages.append(name)
-                                item.status().setToBeInstalled (zypp.ResStatus.USER)
-                            break
-                    else:
-                        if name == sp[0]:
-                            found = True
-                            if name not in self.packages:
-                                self.packages.append(name)
-                                item.status().setToBeInstalled (zypp.ResStatus.USER)
+                            markPoolItem(name, item)
                             break
                 else:
-                    if name in self.incpkgs or self.excpkgs:
-                        found =  True
-                        continue
-                    if startx and name.endswith(sp[0][1:]):
+                    found = True
+                    markPoolItem(name, value[0][1])
+            except KeyError:
+                pass
+        else:
+            if startx:
+                for key, value in self.pkgstable.items():
+                    if key.endswith(name[1:]):
                         found = True
-                        if name not in self.packages:
-                            self.packages.append(name)
-                            item.status().setToBeInstalled (zypp.ResStatus.USER)
-
-                    if endx and name.startswith(sp[0][:-1]):
+                        markPoolItem(key, value[0][1])
+            if endx:
+                for key, value in self.pkgstable.items():
+                    if key.startswith(name[:-1]):
                         found = True
-                        if name not in self.packages:
-                            self.packages.append(name)
-                            item.status().setToBeInstalled (zypp.ResStatus.USER)
+                        markPoolItem(key, value[0][1])
         if found:
             return None
         else:
@@ -169,82 +164,44 @@ class Zypp:
 
     def deselectPackage(self, pkg):
         """Deselect package.  Can be specified as name.arch or name*"""
-        
+        def markPoolItem(name, poolitem):
+            if poolitem.status().isToBeInstalled():
+                poolitem.status().resetTransact(zypp.ResStatus.USER)
+            if name in self.packages:
+                self.packages.remove(name)
+
         if not self.Z:
             self.__initialize_zypp()
-        
+
         startx = pkg.startswith("*")
         endx = pkg.endswith("*")
         ispattern = startx or endx
         sp = pkg.rsplit(".", 2)
-        for item in self.Z.pool():
-            kind = "%s" % item.kind()
-            if kind == "package":
-                name = "%s" % item.name()
-                if not ispattern:
-                    if len(sp) == 2:
+        name = sp[0]
+        if not ispattern:
+            try:
+                value = self.pkgstable[name]
+                if len(sp) == 2:
+                    for ver, item in value:
                         arch = "%s" % item.arch()
-                        if name == sp[0] and arch == sp[1]:
-                            if item.status().isToBeInstalled():
-                                item.status().resetTransact(zypp.ResStatus.USER)
-                            if name in self.packages:
-                                self.packages.remove(name)
+                        if arch == sp[1]:
+                            markPoolItem(name, item)
                             break
-                    else:
-                        if name == sp[0]:
-                            if item.status().isToBeInstalled():
-                                item.status().resetTransact(zypp.ResStatus.USER)
-                            if name in self.packages:
-                                self.packages.remove(name)
-                            break                             
                 else:
-                    if startx and name.endswith(sp[0][1:]):
-                        if item.status().isToBeInstalled():
-                            item.status().resetTransact(zypp.ResStatus.USER)
-                        if name in self.packages:
-                            self.packages.remove(name)
+                    markPoolItem(name, value[0][1])
+            except KeyError:
+                pass
+        else:
+            if startx:
+                for key, value in self.pkgstable.items():
+                    if key.endswith(name[1:]):
+                        markPoolItem(key, value[0][1])
+            if endx:
+                for key, value in self.pkgstable.items():
+                    if key.startswith(name[:-1]):
+                        markPoolItem(key, value[0][1])
 
-                    if endx and name.startswith(sp[0][:-1]):
-                        if item.status().isToBeInstalled():
-                            item.status().resetTransact(zypp.ResStatus.USER)
-                        if name in self.packages:
-                            self.packages.remove(name)
-    
-    def __selectIncpkgs(self):        
-        found = False
-        for pkg in self.incpkgs:
-            for item in self.Z.pool():
-                kind = "%s" % item.kind()
-                if kind == "package":
-                    name = "%s" % item.name()
-                    repoalias = "%s" % item.repoInfo().alias()
-                    if name == pkg and repoalias.endswith("include"):
-                        found = True
-                        if name not in self.packages:
-                            self.packages.append(name)
-                            item.status().setToBeInstalled (zypp.ResStatus.USER)
-                        break         
-        if not found:
-            raise CreatorError("Unable to find package: %s" % (pkg,))
-    
-    def __selectExcpkgs(self):    
-        found = False        
-        for pkg in self.excpkgs:
-            for item in self.Z.pool():
-                kind = "%s" % item.kind()
-                if kind == "package":
-                    name = "%s" % item.name()
-                    repoalias = "%s" % item.repoInfo().alias()
-                    if name == pkg and not repoalias.endswith("exclude"):
-                        found = True
-                        if name not in self.packages:
-                            self.packages.append(name)
-                            item.status().setToBeInstalled (zypp.ResStatus.USER)
-                        break                     
-        if not found:
-            raise CreatorError("Unable to find package: %s" % (pkg,))
 
-        
     def selectGroup(self, grp, include = pykickstart.parser.GROUP_DEFAULT):
         if not self.Z:
             self.__initialize_zypp()
@@ -260,7 +217,7 @@ class Zypp:
                         self.patterns.append(name)
                         item.status().setToBeInstalled (zypp.ResStatus.USER)
                     break
-                
+
         if found:
             if include == pykickstart.parser.GROUP_REQUIRED:
                 map(lambda p: self.deselectPackage(p), grp.default_packages.keys())
@@ -440,10 +397,10 @@ class Zypp:
         repo_alias = repo.id
         if inc:
             repo_alias = name + "include"
-            self.incpkgs = inc
+            self.incpkgs += inc
         if exc:
             repo_alias = name + "exclude"
-            self.excpkgs = exc
+            self.excpkgs += exc
 
         # check LICENSE files
         if not self.checkRepositoryEULA(name, repo):
@@ -475,12 +432,36 @@ class Zypp:
     def installHasFile(self, file):
         return False
 
+    def __fillPkgsTable(self):
+        def add_item(name, value):
+            if not name in self.pkgstable.keys():
+                self.pkgstable[name] = [value]
+            else:
+                self.pkgstable[name].append(value)
+
+        self.incpkgs = set(self.incpkgs)
+        self.excpkgs = set(self.excpkgs)
+        for item in self.Z.pool():
+            kind = "%s" % item.kind()
+            if kind == "package":
+                name = "%s" % item.name()
+                ver = "%s-%s" % (item.edition().version(), item.edition().release())
+                value = (ver,item)
+                repoalias = "%s" % item.repoInfo().alias()
+
+                if name in self.incpkgs:
+                    if repoalias.endswith("include"):
+                        add_item(name, value)
+                elif name in self.excpkgs:
+                    if not repoalias.endswith("exclude"):
+                        add_item(name, value)
+                else:
+                    add_item(name, value)
+
+        for pkg in self.pkgstable.keys():
+            self.pkgstable[pkg].sort(key = lambda x:("%s" %x[1].edition().epoch(), x[0]), reverse =True)
+
     def runInstall(self, checksize = 0):
-        if self.incpkgs:
-            self.__selectIncpkgs()
-        if self.excpkgs:
-            self.__selectExcpkgs()
-        
         os.environ["HOME"] = "/"
         self.buildTransaction()
 
@@ -501,7 +482,7 @@ class Zypp:
         if self.__recording_pkgs:
             # record all pkg and the content
             for pkg in dlpkgs:
-                pkg_long_name = "%s-%s.%s.rpm" % (pkg.name(), pkg.edition(), pkg.arch())
+                pkg_long_name = "%s.%s %s" % (pkg.name(), pkg.arch(), pkg.edition())
                 self.__pkgs_content[pkg_long_name] = {} #TBD: to get file list
 
         total_count = len(dlpkgs)
@@ -523,7 +504,7 @@ class Zypp:
         try:
             self.downloadPkgs(dlpkgs)
             self.installPkgs(dlpkgs)
-    
+
         except RepoError, e:
             raise CreatorError("Unable to download from repo : %s" % (e,))
         except RpmError, e:
@@ -541,7 +522,7 @@ class Zypp:
         shutil.rmtree(self.creator.cachedir + "/etc", ignore_errors = True)
         shutil.rmtree(self.creator.cachedir + "/raw", ignore_errors = True)
         shutil.rmtree(self.creator.cachedir + "/solv", ignore_errors = True)
-        
+
         zypp.KeyRing.setDefaultAccept( zypp.KeyRing.ACCEPT_UNSIGNED_FILE
                                        | zypp.KeyRing.ACCEPT_VERIFICATION_FAILED
                                        | zypp.KeyRing.ACCEPT_UNKNOWNKEY
@@ -553,7 +534,7 @@ class Zypp:
         self.repo_manager_options.repoRawCachePath = zypp.Pathname(self.creator.cachedir + "/raw")
         self.repo_manager_options.repoSolvCachePath = zypp.Pathname(self.creator.cachedir + "/solv")
         self.repo_manager_options.repoPackagesCachePath = zypp.Pathname(self.creator.cachedir + "/packages")
-        
+
         self.repo_manager = zypp.RepoManager(self.repo_manager_options)
 
 
@@ -588,7 +569,7 @@ class Zypp:
             elif self.creator.target_arch == "armv7nhl":
                 arch_map["armv7nhl"] = zypp.Arch_armv7nhl()
             elif self.creator.target_arch == "armv7hl":
-                arch_map["armv7hl"] = zypp.Arch_armv7hl() 
+                arch_map["armv7hl"] = zypp.Arch_armv7hl()
             zconfig.setSystemArchitecture(arch_map[self.creator.target_arch])
 
         print "zypp architecture: %s" % zconfig.systemArchitecture()
@@ -604,11 +585,14 @@ class Zypp:
                 self.repo_manager.buildCache( repo, zypp.RepoManager.BuildIfNeeded )
             else:
                 self.repo_manager.refreshMetadata(repo, zypp.RepoManager.BuildIfNeeded)
-            self.repo_manager.loadFromCache( repo );
+            self.repo_manager.loadFromCache( repo )
 
         self.Z = zypp.ZYppFactory_instance().getZYpp()
         self.Z.initializeTarget( zypp.Pathname(self.creator._instroot) )
-        self.Z.target().load();
+        self.Z.target().load()
+
+        """ Save info of all the packages to self.pkgstable """
+        self.__fillPkgsTable()
 
 
     def buildTransaction(self):
@@ -655,7 +639,7 @@ class Zypp:
             proxies = {}
             if proxy:
                 proxies = {str(proxy.split(":")[0]):str(proxy)}
-          
+ 
             location = zypp.asKindPackage(po).location()
             location = location.filename().__str__()
             if location.startswith("./"):
