@@ -90,6 +90,12 @@ def get_repo_metadata(repo, cachedir, reponame, proxies = {}, arch = None):
     return primarydb
 
 def gen_sqlitedb_from_xml(primaryxml, sqlitedb, target_arch = None):
+    def get_text(element, default = ""):
+        if element.text is None:
+            return default
+        else:
+            return element.text
+
     con = sqlite.connect(sqlitedb)
     con.executescript("""
         DROP table IF EXISTS packages;
@@ -116,33 +122,24 @@ def gen_sqlitedb_from_xml(primaryxml, sqlitedb, target_arch = None):
             myrequires = []
             for subelement in element.getchildren():
                 if subelement.tag.endswith("name"):
-                    pkgname = subelement.text
+                    pkgname = get_text(subelement)
                 elif subelement.tag.endswith("arch"):
-                    arch = subelement.text
+                    arch = get_text(subelement)
                 elif subelement.tag.endswith("version"):
                     epoch = subelement.get("epoch", "")
                     version = subelement.get("ver", "")
                     release = subelement.get("rel", "")
                 elif subelement.tag.endswith("checksum"):
-                    pkgId = subelement.text
+                    pkgId = get_text(subelement)
                     checksum_type = subelement.get("type", "")
                 elif subelement.tag.endswith("summary"):
-                    summary = subelement.text
+                    summary = get_text(subelement)
                 elif subelement.tag.endswith("description"):
-                    if subelement.text:
-                        description = subelement.text
-                    else:
-                        description = summary
+                    description = get_text(subelement, summary)
                 elif subelement.tag.endswith("packager"):
-                    if subelement.text:
-                        rpm_packager = subelement.text
-                    else:
-                        rpm_packager = ""
+                    rpm_packager = get_text(subelement)
                 elif subelement.tag.endswith("url"):
-                    if subelement.text:
-                        url = subelement.text
-                    else:
-                        url = ""
+                    url = get_text(subelement)
                 elif subelement.tag.endswith("time"):
                     time_file = subelement.get("file", "")
                     time_build = subelement.get("build", "")
@@ -156,18 +153,15 @@ def gen_sqlitedb_from_xml(primaryxml, sqlitedb, target_arch = None):
                 elif subelement.tag.endswith("format"):
                     for subsubelement in subelement.getchildren():
                         if subsubelement.tag.endswith("license"):
-                            rpm_license = subsubelement.text
+                            rpm_license = get_text(subsubelement)
                         elif subsubelement.tag.endswith("vendor"):
-                            rpm_vendor = subsubelement.text
+                            rpm_vendor = get_text(subsubelement)
                         elif subsubelement.tag.endswith("group"):
-                            rpm_group = subsubelement.text
+                            rpm_group = get_text(subsubelement)
                         elif subsubelement.tag.endswith("buildhost"):
-                            rpm_buildhost = subsubelement.text
+                            rpm_buildhost = get_text(subsubelement)
                         elif subsubelement.tag.endswith("sourcerpm"):
-                            if subsubelement.text:
-                                rpm_sourcerpm = subsubelement.text
-                            else:
-                                rpm_sourcerpm = ""
+                            rpm_sourcerpm = get_text(subsubelement)
                         elif subsubelement.tag.endswith("header-range"):
                             rpm_header_start = subsubelement.get("start", "")
                             rpm_header_end = subsubelement.get("end", "")
@@ -509,8 +503,7 @@ def is_mainrepo(cachedir, reponame):
     retval = False
     if has_package_in_repo(con, "libzypp") \
        and has_package_in_repo(con, "mic2") \
-       and has_package_in_repo(con, "rpm") \
-       and has_package_in_repo(con, "fastinit"):
+       and has_package_in_repo(con, "rpm"):
         retval = True
 
     con.close()
