@@ -180,8 +180,6 @@ def ismeego(rootdir):
     ret = False
     if (os.path.exists(rootdir + "/etc/moblin-release") \
        or os.path.exists(rootdir + "/etc/meego-release")) \
-       and os.path.exists(rootdir + "/etc/inittab") \
-       and os.path.exists(rootdir + "/etc/rc.sysinit") \
        and glob.glob(rootdir + "/boot/vmlinuz-*"):
         ret = True
 
@@ -730,7 +728,7 @@ def get_package(pkg, repometadata, arch = None):
 def get_source_name(pkg, repometadata):
 
     def get_bin_name(pkg):
-        m = re.match("(.*)-(.*)-(.*)\.(.*)\.rpm", pkg)
+        m = re.match("(.*)\.(.*) (.*)-(.*)", pkg)
         if m:
             return m.group(1)
         return None
